# ifindron - calculates readout noise of detector
# Author : Robert Greimel (greimel@ing.iac.es)
# Version: 23. Aug. 2000
# History: Based on midas script by Thomas Augusteijn
##############################################################################
procedure ifindron (dark1,dark2)

string	dark1			{prompt="First dark frame"}
string	dark2			{prompt="Second dark frame"}

string	outtab			{prompt="Name of output table"}

string	section = "[*,*]"	{prompt="Selected image section"}
int	xdiv			{prompt="number of divisions in x"}
int	ydiv			{prompt="number of divisions in y"}

real	clip = 3		{prompt="clipping factor in sigma"}

string	center = "mean"		{prompt="Central statistical measure",
				    enum="mean|midpt|mode"}
real	binwidth = 0.1		{prompt="Bin width of histogram in sigma"}

bool	plot = yes		{prompt="plot the result ?"}
bool	verbose = yes		{prompt="Verbose output?"}

string	*list1
string	*list2

begin
  string	darkimg,indark1,indark2,trim,table,name
  string	stats1,stats2,stmp1,stmp2
  int		ip,xs,xe,xdp,ys,ye,ydp,i,j,mndr,coadd
  real		low,high,ron,ronerr,avg,avgerr,rtmp1,rtmp2,md1,md2

  # need to copy arguments to define them for sure
  indark1 = dark1
  indark2 = dark2

  # check if output table already exists
  if ( outtab != "" && access(outtab) )
    {
    beep
    print("")
    print("ERROR: operation would override output table "//outtab)
    print("ifindron aborted")
    beep
    bye
    }

  # find exposure information for first dark image
  ifileinfo(input=indark1, verbose=no)
  rtmp1 = ifileinfo.exptim
  coadd = ifileinfo.coadd
  mndr = ifileinfo.mndr

  # find exposure information for second dark image
  ifileinfo(input=indark2, verbose=no)

  # check if exposure times are the same. If not, print a warning
  if ( ifileinfo.exptim != rtmp1 )
    {
    print(" ")
    print("WARNING: exposure times for frames is different")
    print(" ")
    print("         first  dark: "//rtmp1)
    print("         second dark: "//ifileinfo.exptim)
    print(" ")
    }

  # check that number of coadds are the same
  if ( ifileinfo.coadd != coadd )
    {
    beep
    print("")
    print("ERROR: number of coadds for frames is different")
    print("")
    print("         first  dark: "//coadd)
    print("         second dark: "//ifileinfo.coadd)
    print("")
    print("ifindron aborted")
    beep
    bye
    }

  # check that number of coadds is > 0
  if ( coadd < 1 )
    {
    print(" ")
    print("WARNING: number of coadds is "//coadd//" - should be >= 1")
    print("         adjusting to 1")
    print(" ")
    coadd = 1
    }

  # check that number of MNDR are the same
  if ( ifileinfo.mndr != mndr )
    {
    beep
    print("")
    print("ERROR: number of MNDR for frames is different")
    print("")
    print("         first  dark: "//mndr)
    print("         second dark: "//ifileinfo.mndr)
    print("")
    print("ifindron aborted")
    beep
    bye
    }

  # check that number of MNDR is 1
  if ( mndr != 1 )
    {
    print(" ")
    print("WARNING: number of MNDR is "//mndr//" - should be 1")
    printf("         The result will be wrong (about %5.2f to small) !",sqrt(mndr))
    print(" ")
    }

  # subtract the dark images from each other
  # isubtract also guarantees that we will get a post-pre subtracted image back

  darkimg = mktemp ("tmp$ifindron")
  isubtract (inputr1 = indark1,
             inputr2 = indark2,
             output = darkimg )
#imcopy(indark2, darkimg)

  # get size of useful area from image header if the section was not
  # specified directly
  ip = stridx("[",section)
  if ( ip == 0 )
    {
    imgets (image=darkimg, param=section)
    trim = imgets.value
    }
  else
    {
    trim = section
    }

  # extract section bounding values
  # first, split the x and y sections
  # then expand * if necessary
  # last, assign the section start and end values
  ip = stridx(",", trim)
  stmp1 = substr(trim, 2, ip-1)
  stmp2 = substr(trim, ip+1, strlen(trim)-1)

  ip = stridx(":", stmp1)
  if ( ip == 0 )
    {
    xs = 1
    imgets (image=darkimg, param="i_naxis1")
    xe = int(imgets.value)
    }
  else
    {
    xs = int(substr(stmp1,1,ip-1))
    xe = int(substr(stmp1,ip+1,strlen(stmp1)))
    }
  xdp = int( (xe-xs+1) / xdiv )
  if ( verbose ) {print ( "x step size : "//xdp//" pixels" )}

  ip = stridx(":", stmp2)
  if ( ip == 0 )
    {
    ys = 1
    imgets (image=darkimg, param="i_naxis2")
    ye = int(imgets.value)
    }
  else
    {
    ys = int(substr(stmp2,1,ip-1))
    ye = int(substr(stmp2,ip+1,strlen(stmp2)))
    }
  ydp = int( (ye-ys+1) / ydiv )
  if ( verbose ) {print ( "y step size : "//ydp//" pixels" )}

  # create temporary file to hold list of image sections to find
  # statistics for
  stmp1 = mktemp ("tmp$ifindron")

  # loop over x divisions
  for (i=1; i<=xdiv; i+=1)
    {
    stmp2 = darkimg//"["//(xs+(i-1)*xdp)//":"//(xs+i*xdp-1)//","

    # loop over y divisions
    for (j=1; j<=ydiv; j+=1)
      {
      print (stmp2//(ys+(j-1)*ydp)//":"//(ys+j*ydp-1)//"]", >> stmp1)
      }
    }

  # calculate image statistics for all sections
  if ( verbose ) {print ("calculating statistics ...")}
  stats1 = mktemp ("tmp$ifindron")
  imstatistics ("@"//stmp1, \
                fields=center//",stddev", \
                lower=INDEF, \
                upper=INDEF, \
                binwidth=binwidth, \
                format=no, > stats1)

  # re-calculate image statistics for all sections
  if (verbose )
    {print ("recalculating statistics with "//clip//" sigma clipping ...")}

  stats2 = mktemp ("tmp$ifindron")
  list1 = stmp1
  list2 = stats1
  while ( fscan(list1, name) == 1 )
    {
    if (fscan (list2, rtmp1, rtmp2) != 2) goto err
    low = rtmp1 - clip*rtmp2
    high = rtmp1 + clip*rtmp2
    imstatistics (name, \
                  fields =center//",stddev", \
                  lower = low, \
                  upper = high, \
                  binwidth = binwidth, \
                  format = no, >> stats2)
    }

err:
  list1 = ""
  list2 = ""

  # read statistics into table
  table = mktemp ("tmp$ifindron")//".tab"
  tcreate (table = table, \
           cdfile = "ingrid_ql$ifindron.cd", \
           datafile = stats2, \
           uparfile = "", \
           nskip = 0, \
           nlines = 0, \
           nrows = 0, \
           hist = no, \
           extrapar = 0, \
           tbltype = "default", \
           extracol = 0 )

  # adjust standard deviation for number of MNDRs and the fact that we used a
  # difference of two images
  tcalc (table = table,
         outcol = "stdev",
         equals = "stdev*sqrt("//coadd//"/2.)")

  # calculate final statistics
  tstat (intable = table, \
         column = "stdev", \
         outtable = "", \
         lowlim = INDEF,\
         highlim = INDEF )

  # find limits for recalculation of final statistics
  if ( center == "mean" )
    {
    low = tstat.mean - 2*tstat.stddev
    high = tstat.mean + 2*tstat.stddev
    }
  else
    {
    low = tstat.median - 2*tstat.stddev
    high = tstat.median + 2*tstat.stddev
    }

  tstat (intable = table, \
         column = "stdev", \
         outtable = "", \
         lowlim = low,\
         highlim = high )

  if ( center == "mean" ) {ron = tstat.mean}
  else                    {ron = tstat.median}
  ronerr = tstat.stddev / sqrt(tstat.nrows - 1)

  tstat (intable = table, \
         column = "mean", \
         outtable = "", \
         lowlim = INDEF,\
         highlim = INDEF )

  if ( center == "mean" ) {avg = tstat.mean}
  else                    {avg = tstat.median}
  avgerr = tstat.stddev

  # find median values for first input images
  istatistics (indark1//trim, \
               sections="data", \
               fields="midpt", \
               lower=INDEF, \
               upper=INDEF, \
               binwidth=0.1, \
               format=no) | scanf("%f",md1)

  # find median values for second input images
  istatistics (indark2//trim, \
               sections="data", \
               fields="midpt", \
               lower=INDEF, \
               upper=INDEF, \
               binwidth=0.1, \
               format=no) | scanf("%f",md2)

  # print results
  printf ("median of first dark  : %6.3f\n",md1)
  printf ("median of second dark : %6.3f\n",md2)
  printf ("average               : %6.3f +- %5.3f\n",avg,avgerr)
  printf ("readnoise             : %6.3f +- %5.3f\n",ron,ronerr)

  # plot results if requested
  if ( plot )
    {
    axispar.xlabel = "Standard Deviation"
    axispar.ylabel = "Number"
    thistogram (table,"STDOUT","stdev") | sgraph (crvstyle="pseudohist")
    }

  # delete temporary files
  imdelete(darkimg,yes,verify=no)
  if ( outtab == "" )
    {
    delete(table,yes,verify=no)
    }
  else
    {
    rename(table,outtab,field="all")
    }
  delete(stats1,yes,verify=no)
  delete(stats2,yes,verify=no)
  delete(stmp1,yes,verify=no)

end
